/*
 * @(#)CommandMenu.java 5.1
 *
 */

package CH.ifa.draw.util;

import java.awt.*;
import java.util.*;

/**
 * A Command enabled menu. Selecting a menu item
 * executes the corresponding command.
 *
 * @see Command
 */

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public  class CommandMenu
        extends Menu implements ActionListener {

    public CommandMenu(String name) {
        super(name);
    }

    private void addCommandMenuItem(CommandMenuItem m) {
	m.setFont(getFont());
        // m.setName(command.name()); necessary??
        m.addActionListener(this);
        add(m);
    }

    /**
     * Adds a command to the menu. The item's label is
     * the command's name.
     */
    public synchronized void add(Command command) {
        addCommandMenuItem(new CommandMenuItem(command));
    }

    /**
     * Adds a command with the given short cut to the menu. The item's label is
     * the command's name.
     */
    public synchronized void add(Command command, MenuShortcut shortcut) {
        addCommandMenuItem(new CommandMenuItem(command, shortcut));
    }

    /**
     * Adds a separator to the menu at the current position.
     * The separator is disabled (as promised but not 
     * implemented by java.awt.Menu).
     */
    public void addSeparator() {
      MenuItem item = new MenuItem("-");
      item.setEnabled(false);
      super.add(item);
    }

    public synchronized void remove(Command command) {
        System.out.println("CommandMenu.remove() not implemented");
    }

    public synchronized void remove(MenuItem item) {
        System.out.println("CommandMenu.remove() not implemented");
    }

    /**
     * Changes the enabling/disabling state of a named menu item.
     */
    public synchronized void enable(String name, boolean state) {
        for (int i = 0; i < getItemCount(); i++) {
            MenuItem item = getItem(i);
            if (name.equals(item.getLabel())) {
                item.setEnabled(state);
                return;
            }
        }
    }

    public synchronized boolean checkEnabled() {
        boolean menuEnabled = false;
	boolean enabled;

        for (int i = 0; i < getItemCount(); i++) {
            MenuItem item = getItem(i);
            if (item instanceof CommandMenuItem) {
		enabled = ((CommandMenuItem)item).cmd.isExecutable();
		item.setEnabled(enabled);
		menuEnabled = menuEnabled | enabled;
	    } else if (item instanceof CommandMenu) {
  	        enabled = ((CommandMenu)item).checkEnabled();
		item.setEnabled(enabled);
		menuEnabled = menuEnabled | enabled;
	    } else {
  	        menuEnabled = menuEnabled | item.isEnabled();
	    }
        }
	return menuEnabled;
    }

    /**
     * Executes the command.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
	if (source instanceof CommandMenuItem) {
	    Command cmd = ((CommandMenuItem)source).cmd;
	    cmd.execute();
        }
    }
}

class CommandMenuItem extends MenuItem {

    final Command cmd;

    CommandMenuItem(Command cmd) {
	super(cmd.name());
	this.cmd = cmd;
    }

    CommandMenuItem(Command cmd, MenuShortcut ms) {
	super(cmd.name(),ms);
	this.cmd = cmd;
    }

}
