/*
 * @(#)StorableInOut.java 5.1
 *
 */

package CH.ifa.draw.util;

import java.util.*;
import java.io.*;
import de.renew.util.StringUtil;

/**
 * An abstract superclass of storable input and output streams.
 * StorableInOut handles the object identity of the stored objects
 * through a map and preserves the original filename.
 *
 * @see Storable
 * @see StorableInput
 * @see StorableOutput
 */


public abstract class StorableInOut extends Object {

    private Vector fMap = new Vector();
    private String path=null;

    protected StorableInOut() { }

    protected StorableInOut(String filename) {
	if (filename!=null) {
	    try {
		path=StringUtil.getPath(new File(filename).getCanonicalPath());
	    } catch (IOException e) {
		System.err.println("Could not find file "
				   +filename+": "+e);
	    }
	}
	//if (path!=null) {
	//    System.out.println("Setting base path to "+path);
	//}
    }

    public String getPath() {
	return path;
    }

    protected boolean mapped(Storable storable) {
        return fMap.contains(storable);
    }

    protected void map(Storable storable) {
        if (!fMap.contains(storable))
            fMap.addElement(storable);
    }

    protected Storable retrieve(int ref) {
        return (Storable) fMap.elementAt(ref);
    }

    protected int getRef(Storable storable) {
	return fMap.indexOf(storable);
    }

}
