package de.renew.debug;

import de.renew.database.*;
import java.io.PrintStream;
import de.renew.simulator.*;

/**
 * A transaction factory is responsible for creating
 * transactions that control the access to a net database.
 * 
 * The factory allows applications to configure individual databases
 * depending on the given environment.
 */

public class TracingTransactionStrategy implements TransactionStrategy {
  private PrintStream stream;
  private int count=0;

  public TracingTransactionStrategy(PrintStream stream) {
    this.stream=stream;
  }

  private String makePrefix() {
    return "<<"+(count++)+">>";
  }

  public void perform(NetAction[] createActions,
		      TokenAction[] addActions,
		      TokenAction[] removeActions,
		      NetAction[] deleteActions)
  {
    String prefix=makePrefix();

    for (int i=0;i<createActions.length;i++) {
      stream.println(prefix+" Created net "+createActions[i].getName()+
		     "["+createActions[i].getNetID()+"]");
    }

    for (int i=0;i<addActions.length;i++) {
      stream.println(prefix+" Put "+addActions[i].getToken()+
		     " with ID "+addActions[i].getTokenID()+
		     " into "+addActions[i].getNetID()+
		     ":"+addActions[i].getPlaceID());
    }

    for (int i=0;i<removeActions.length;i++) {
      stream.println(prefix+" Removed "+removeActions[i].getToken()+
		     " with ID "+removeActions[i].getTokenID()+
		     " from "+removeActions[i].getNetID()+
		     ":"+removeActions[i].getPlaceID());
    }

    for (int i=0;i<deleteActions.length;i++) {
      stream.println(prefix+" Deleted net "+deleteActions[i].getName()+
		     "["+deleteActions[i].getNetID()+"]");
    }
  }
}
