package de.renew.formalism.fs;

import de.renew.shadow.SyntaxException;
import de.renew.shadow.*;
import collections.*;
import java.util.Enumeration;
import de.uni_hamburg.fs.*;
import de.renew.formalism.java.*;

public class FSNetCompiler extends JavaNetCompiler implements FSShadowCompiler {
  static final long serialVersionUID = 262012294953452116L;

  public FSNetCompiler() {
    super(true,false,true);
  }

  protected InscriptionParser makeParser(String inscr) {
    return new FSNetParser(new java.io.StringReader(inscr));
  }

  protected CollectionEnumeration parseArcInscription(String inscr,
    ParsedDeclarationNode declaration) throws SyntaxException
  {
    if (inscr==null || inscr.equals("")) {
      inscr="[]";
    }
    return super.parseArcInscription(inscr,declaration);
  }

  public void checkConcept(String conceptName, String appropStr, ShadowNet shadowNet)
    throws SyntaxException {
 }

  // Compile a net system and return a lookup that knows
  // about the compiled nets.
  public synchronized ShadowLookup compile(ShadowNetSystem netSystem)
    throws SyntaxException {
       return compile(new FSShadowLookup(), netSystem);
  }

  protected void createNetElements() throws SyntaxException {
     ShadowNetSystem netSystem=getNetSystem();
     Partition topPartition=new Partition();
     // compile the type hierarchy...
     // First, remove all (old) user concepts:
     de.uni_hamburg.fs.TypeSystem.newInstance();
     // second, compile all concepts from all drawings:
    CollectionEnumeration enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compileConcepts((ShadowNet)enum.nextElement(),topPartition);
    }
    // then, add all "approp"-information:
    enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compileApprops((ShadowNet)enum.nextElement());
    }
    // then, add all "isNotA"-information:
    enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compilePartitions((ShadowNet)enum.nextElement());
    }
    // last, add all "isa"-information:
    enum=netSystem.elements();
    while (enum.hasMoreElements()) {
      compileIsas((ShadowNet)enum.nextElement());
    }
    // and now:
    try {
	TypeSystem.instance().inheritFeatures();
    } catch (TypeException tex) {
	SyntaxException ex=
	    new SyntaxException("Feature \""+tex.featureName
				+"\" has been redefined illegally in concept \""
				+tex.concept.getName()+"\".",
				null);
	FSShadowLookup lookup=(FSShadowLookup)getLookup();
	ex.addObject(lookup.getApprop(tex.concept,tex.featureName));
	throw ex;
    }
    // all additional work has been done.
    // continue in superclass:
    super.createNetElements();
  }

  private void compileConcepts(ShadowNet net,
			       Partition topPartition) throws SyntaxException {
   FSShadowLookup lookup=(FSShadowLookup)getLookup();
   int counter=0;
   CollectionEnumeration enum=net.elements();
   while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowConcept) {
	  ShadowConcept shadowConcept=(ShadowConcept)elem;
	  String conceptName=shadowConcept.getName();
System.out.println("Compiling concept "+conceptName+"...");
	  // Concept with that name may already exist:
	  if (TypeSystem.instance().hasConcept(conceptName)) {
	      SyntaxException ex=
		  new SyntaxException("The concept \""
				      +conceptName+"\" already exists.",
				      null);
	      ex.addObject(shadowConcept);
	      throw ex;
	  }
	  ConceptImpl concept=new ConceptImpl(conceptName);
	  lookup.setConcept(shadowConcept,concept);
	  if (!shadowConcept.elements().hasMoreElements()) {
	      // is this a top level concept?
	      try {
		  topPartition.addConcept(concept);
	      } catch (TypeException tex) {
		  // should never happen!
		  throw new RuntimeException(tex.toString());
	      }
	  }
	  //System.out.println("Created Concept "+conceptName);
      }
    }
  }

  private void compileApprops(ShadowNet net) throws SyntaxException {
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    CollectionEnumeration enum=net.elements();
    while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowConcept) {
        ShadowConcept shadowConcept=(ShadowConcept)elem;
        ConceptImpl concept=lookup.getConcept(shadowConcept);
        // parse lines:
        FSNetParser parser=(FSNetParser)makeParser("\n"+shadowConcept.getApprop());
        parser.setLookup(lookup);
        //parser.setDeclarationNode(declaration);
        try {
	    OrderedTable approps=parser.parseAppropDef();
	    concept.setApprops(approps);
	    Enumeration appropenum=approps.keys();
	    while (appropenum.hasMoreElements()) {
		Name featureName=(Name)appropenum.nextElement();
		lookup.setApprop(concept,featureName,
				 shadowConcept);
	    }
        } catch (de.renew.formalism.java.ParseException e) {
          SyntaxException ex=makeSyntaxException(e);
          ex.addObject(shadowConcept);
          throw ex;
        }
      }
    }
    enum=net.elements();
    while (enum.hasMoreElements()) {
	ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
	if (elem instanceof ShadowAssoc) {
	    ShadowAssoc shadowAssoc=(ShadowAssoc)elem;
	    ConceptImpl concept=lookup.getConcept(shadowAssoc.getConcept());
	    ConceptSet typ=new ConceptSet(lookup.getConcept(shadowAssoc.getType()));
	    Name featureName=shadowAssoc.getFeature();
	    concept.addApprop(featureName,typ);
	    lookup.setApprop(concept,featureName,shadowAssoc);
    //System.out.println("Found Assoc: "+shadowAssoc.getFeature()+": "+typ);
	}
    }
  }

  private void compilePartitions(ShadowNet net) throws SyntaxException {
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    CollectionEnumeration enum=net.elements();
    while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowIsa) {
        ShadowIsa shadowIsa=(ShadowIsa)elem;
	if (shadowIsa.isDisjunctive()) {
	    ConceptImpl sub=lookup.getConcept(shadowIsa.getSource());
	    ShadowConcept supi=shadowIsa.getTarget();
System.out.println("Adding "+sub.getName()+" to sub-partition of "+supi.getName());
	    try {
		lookup.addToSubPartition(supi,sub);
	    } catch (TypeException tex) {
		SyntaxException ex=new SyntaxException("This disjunction leads to a contradiction.",null);
		ex.addObject(shadowIsa);
		throw ex;
	    }
	}
	    
      }
    }
  }

  private void compileIsas(ShadowNet net) throws SyntaxException {
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    CollectionEnumeration enum=net.elements();
    while (enum.hasMoreElements()) {
      ShadowNetElement elem=(ShadowNetElement)enum.nextElement();
      if (elem instanceof ShadowIsa) {
        ShadowIsa shadowIsa=(ShadowIsa)elem;
        ConceptImpl sub=lookup.getConcept(shadowIsa.getSource());
	ShadowConcept supi=shadowIsa.getTarget();
        ConceptImpl sup=lookup.getConcept(supi);
System.out.println("Compiling is-a from "+sub.getName()+" to "+sup.getName());
        try {
          sub.addIsa(sup);
        } catch (CyclicHierarchyException e) {
          SyntaxException ex=new SyntaxException("This is-a-relation leads to a cyclic type hierarchy.",null);
          ex.addObject(shadowIsa);
          throw ex;
        } catch (TypeException te) {
	    SyntaxException ex=new SyntaxException("This is-a-relation leads to a contradiction.",null);
	    ex.addObject(shadowIsa);
	    throw ex;
	}
      }
    }
  }


}
