package de.renew.formalism.fs;

import java.util.Vector;
import de.uni_hamburg.fs.*;
import de.renew.unify.*;
import collections.*;

class FSUnifier {
  Variable result;
  private Vector variables= new Vector();
  private FeatureStructure current;
  private int open=0;
    //  private boolean resultSet=false;

  FSUnifier(FeatureStructure template, Vector paths, StateRecorder recorder)
    throws Impossible
  {
    result=new Variable();
    current=template;

    // Create listeners.
    for (int i=0; i<paths.size(); i++) {
      Variable variable = new Variable();
      variables.addElement(variable);
      setupListener(variable,(String)paths.elementAt(i),recorder);
      open++;
    }
    setupResultListener(paths,recorder);
  }

  Variable getVariable(int i) {
    return (Variable)variables.elementAt(i);
  }

  private void setupListener(final Variable variable, final String path,
    StateRecorder recorder) throws Impossible
  {
    variable.addListener(new Notifiable() {
      public void boundNotify(StateRecorder irecorder) throws Impossible {
//  	if (resultSet) {
//  	    return;
//  	}
	Object val=variable.getValue();
	final FeatureStructure oldCurrent=current;
	irecorder.record(new StateRestorer() {
	  public void restore() {
	    current=oldCurrent;
	    open++;
	  }
	});
	open--;
	FeatureStructure fs;
	if (val instanceof FeatureStructure) {
	  fs=(FeatureStructure)val;
        } else if (val instanceof Class) {
           fs=new FeatureStructure(new ConjunctiveType(TypeSystem.instance().getJavaConcept((Class)val)));
	} else {
          fs=new FeatureStructure(new JavaObjectFS(val));
	}
	try {
	  current=current.unify(fs,path);
	} catch (UnificationFailure uff) {
	  throw new Impossible();
	}
	if (open==0) {
	  Unify.unify(result,current,irecorder);
	}
      }
    },recorder);
  }

  private void setupResultListener(final Vector paths, StateRecorder recorder)
               throws Impossible {
    result.addListener(new Notifiable() {
      public void boundNotify(StateRecorder irecorder) throws Impossible {
	  Object val=result.getValue();
	  if (!(val instanceof FeatureStructure)) {
	      throw new Impossible();
	  }
	  FeatureStructure resultfs=(FeatureStructure)val;
	  if (!current.subsumes(resultfs)) {
	      // they can never become equal!
	      throw new Impossible();
	  }
//  	  final FeatureStructure oldCurrent=current;
//  	  irecorder.record(new StateRestorer() {
//  	      public void restore() {
//  		  current=oldCurrent;
//  		  resultSet=false;
//  	      }
//  	  });
//  	  resultSet=true;
//  	  try {
//  	      current=((FeatureStructure)val).unify(current);
//  	  } catch (UnificationFailure uff) {
//  	      throw new Impossible();
//  	  }
	  for (int i=0; i<paths.size(); i++) {
	      Variable variable=(Variable)variables.elementAt(i);
	      if (!Unify.isBound(variable)) {
		  String path=(String)paths.elementAt(i);
//		  Unify.unify(variable,current.unpackingAt(path),irecorder);
		  Unify.unify(variable,resultfs.unpackingAt(path),irecorder);
	      }
	  }
      }
    },recorder);
  }
}
