package de.renew.formalism.fs;

import java.util.*;
import de.uni_hamburg.fs.*;
import de.renew.unify.Impossible;
import de.renew.unify.Tuple;

public class MarkingNetInstance extends ObjectNetInstance {

    public MarkingNetInstance(FeatureStructure fs, boolean confirm)
	   throws Impossible {
	super((String)fs.at("net").getJavaObject(),extractMarking(fs));
	if (confirm)
	    confirm();
    }

    public MarkingNetInstance(FeatureStructure fs)
	   throws Impossible {
	this(fs,true);
    }

    public MarkingNetInstance confirm() {
	super.earlyConfirmation();
	super.earlyConfirmationTrace(de.renew.simulator.SilentTracer.INSTANCE);
	super.lateConfirmation(de.renew.simulator.SilentTracer.INSTANCE);
	return this;
    }

    private static Vector vector(Object o) {
	Vector v=new Vector();
	v.addElement(o);
	return v;
    }

    public FeatureStructure equateAndFire(FeatureStructure proc,
					  FeatureStructure transition,
					  String[] paths)
	                    throws UnificationFailure {
	String path0="proc"+paths[0];
	proc=proc.unify(transition,path0);
	for (int i=1; i<paths.length; ++i) {
	    proc=proc.equate(path0,"proc"+paths[i]);
	}
	return proc;
    }

    public FeatureStructure fire(FeatureStructure proc,
				 FeatureStructure transition,
				 Object paths) throws UnificationFailure {
	((FSNode)transition.getRoot()).totallyWellType();
	if (paths instanceof Tuple) {
	    return equateAndFire(proc,transition,
			(String[])((Tuple)paths).asArray(String.class));
	} else {
	    return proc.unify(transition,"proc"+paths);
	}
    }

    private static Hashtable extractMarking(FeatureStructure fs) {
	Hashtable placeMap=new Hashtable();
	FSNode proc=(FSNode)fs.getRoot().delta(new Name("proc"));
	proc.totallyWellType();
	extractMarking(placeMap,new Vector(),"",proc);
	placeMap.put("Process",vector(fs));
	return placeMap;
    }

    private static void extractMarking(Hashtable placeMap, Vector visited,
				       String path, Node curr) {
	if (visited.contains(curr)) // already visited?
	    return;
	visited.addElement(curr);
	Enumeration featenum=curr.featureNames();
	while (featenum.hasMoreElements()) {
	    Name featureName=(Name)featenum.nextElement();
	    String feature=featureName.toString();
	    String newpath=path+":"+feature;
	    Node next=curr.delta(featureName);
	    if (next.featureNames().hasMoreElements()) {
		extractMarking(placeMap, visited, newpath, next);
	    } else {
		// leaf found
		if (placeMap.containsKey(feature)) {
		    ((Vector)placeMap.get(feature)).addElement(newpath);
		} else {
		    placeMap.put(feature,vector(newpath));
		}
	    }
	}
    }

    public static FeatureStructure processMarking(FeatureStructure fs) {
	Type leafType=null, nodeType=null, emptyType=null;
	TypeSystem ts=TypeSystem.instance();
	if (!ts.hasConcept("E") || !ts.hasConcept("T")) {
	    System.out.println("Type E or T not found!");
	    return null;
	}
	FSNode vals=null;
	try {
	    emptyType=ConjunctiveType.getType("E");
	    nodeType=ConjunctiveType.getType("T");
	    if (!ts.hasConcept("Token")) {
		System.out.println("Type Token not found - using type T.");
		leafType=nodeType;
	    } else {
		leafType=ConjunctiveType.getType("Token");
		vals=new FSNode("Marking");
	    }
	} catch (UnificationFailure uff) {
	    uff.printStackTrace();
	}

	try {
	    FSNode proc=(FSNode)fs.getRoot();
	    FSNode state=new FSNode("State");
	    state.addFeature("proc",proc);
	    FSNode mark=new FSNode("Consumers");
	    //System.out.println("Type(mark)="+mark.getType());
	    state.addFeature("mark",mark);
	    if (vals!=null)
		state.addFeature("values",vals);

	    processMarking(new Vector(),proc,mark,vals,leafType,nodeType);

	    Enumeration featenum=mark.getType().appropFeatureNames();
	    while (featenum.hasMoreElements()) {
		Name feat=(Name)featenum.nextElement();
		if (!mark.hasFeature(feat)) {
		    mark.addFeature(feat,new FSNode(emptyType));
		}
	    }

	    return new FeatureStructure(state);
	} catch (TypeException tee) {
	    System.out.println("TypeException while processing marking!");
	    return null;
	} catch (UnificationFailure uff) {
	    return null;
	}
    }

    private static void processMarking(Vector visited,
				       Node curr, FSNode mark,
				       FSNode vals,
				       Type leafType, Type nodeType)
	                throws TypeException {
	if (visited.contains(curr)) // already visited?
	    return;
	visited.addElement(curr);
	Enumeration featenum=curr.featureNames();
	while (featenum.hasMoreElements()) {
	    Name featureName=(Name)featenum.nextElement();
	    if (curr.getType().equals(leafType) && featureName.toString().equals("val"))
		continue;  // do not recurse into tokens!
	    Node next=curr.delta(featureName);
	    Type nextType=next.getType();
	    if (nextType.equals(leafType)) {
		Name valName=new Name("val");
		if (next.hasFeature(valName)) {
		    // coloured leaf:
		    Name consName=new Name("cons");
		    if (!next.hasFeature(consName)) {
			((FSNode)next).addFeature(consName,new FSNode(nodeType));
		    }
		    Node free=next.delta(consName);
		    if (free.getType().equals(nodeType)) {
			// real leaf found
			mark.addFeature(featureName,free);
			// make deep copy of token value:
			//  Node valCopy=null;
//  			try {
//  			    valCopy=next.delta(valName);
//  			    valCopy=EquivRelation.unify
//  				(valCopy,new FSNode(Type.TOP));
//  			    vals.addFeature(featureName,valCopy);
//  			} catch (UnificationFailure uff) {
//  			    System.err.println("***Unexpected unification failure: Trying to add "+featureName+": "+valCopy);
//  			}
			try {
			    vals.addFeature(featureName,next.delta(valName));
			} catch (UnificationFailure uff) {
			    System.err.println("***Unexpected unification failure: Trying to add "+featureName+": "+next.delta(valName));
			}
		    }
		} else {
		    // leaf of B/E net found:
		    mark.addFeature(featureName,next);
		}
	    }
	    if (leafType.subsumes(nextType)
		|| nodeType.subsumes(nextType)) {
		processMarking(visited, next, mark, vals, leafType, nodeType);
	    }
	}
    }

}
