package de.renew.function;

import de.renew.unify.*;
import java.lang.reflect.Constructor;
import de.renew.util.ReflectionSerializer;
import java.io.IOException;

public class ConstructorFunction implements Function {
  /** 
   * This field is not really transient, but as the reflection
   * classes are not serializable, we have to store it by
   * ourselves.
   **/
  transient Constructor constr;

  public ConstructorFunction(Constructor constructor) {
    constr=constructor;
  }

  public Object function(Object param) throws Impossible {
    Tuple tuple=(Tuple)param;

    Object[] paramArr=new Object[tuple.getArity()];
    for (int i=0;i<paramArr.length;i++) {
      paramArr[i]=tuple.getComponent(i);
    }

    try {
      return Executor.executeConstructor((Constructor)constr,paramArr);
    }
    catch (Exception e) {
	if (e instanceof java.lang.reflect.InvocationTargetException) {
	    ((java.lang.reflect.InvocationTargetException)e).getTargetException().printStackTrace();
	}
      e.printStackTrace();
      throw new Impossible(
        "An exception occured during a constructor call.");
    }
  }


  /**
   * Serialization method, behaves like default writeObject
   * method. Stores the not-really-transient constr field.
   **/
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException {
      out.defaultWriteObject();
      ReflectionSerializer.writeConstructor(out, constr);
  }

  /**
   * Deserialization method, behaves like default readObject
   * method. Restores the not-really-transient constr field.
   **/
  private void readObject(java.io.ObjectInputStream in)
      throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      constr = ReflectionSerializer.readConstructor(in);
  }
}

