package de.renew.function;

import java.lang.reflect.*;
import de.renew.unify.*;
import de.renew.util.Value;
import de.renew.util.ReflectionSerializer;
import java.io.IOException;

public class FieldFunction implements Function {

  /** 
   * This field is not really transient, but as the reflection
   * classes are not serializable, we have to store it by
   * ourselves.
   **/
  transient Field field;

  public FieldFunction(Field field) {
    this.field=field;
  }

  public Object function(Object param) throws Impossible {
    try {
      if (param.getClass().isArray()) {
	if (field.equals("length")) {
	  return new Value(new Integer(Array.getLength(param)));
	} else {
	  throw new Impossible();
	}
      }

      return Value.possiblyWrap(field.get(param),
	field.getType().isPrimitive());
    }
    catch (Exception e) {
      throw new Impossible();
    }
  }

  /**
   * Serialization method, behaves like default writeObject
   * method. Stores the not-really-transient field field.
   **/
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException {
      out.defaultWriteObject();
      ReflectionSerializer.writeField(out, field);
  }

  /**
   * Deserialization method, behaves like default readObject
   * method. Restores the not-really-transient field field.
   **/
  private void readObject(java.io.ObjectInputStream in)
      throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      field = ReflectionSerializer.readField(in);
  }

}

