package de.renew.function;

import de.renew.unify.*;
import java.lang.reflect.*;

import de.renew.util.ReflectionSerializer;
import java.io.IOException;

public class StaticMethodFunction implements Function {

  /** 
   * This field is not really transient, but as the reflection
   * classes are not serializable, we have to store it by
   * ourselves.
   **/
  transient Method method;

  public StaticMethodFunction(Method method) {
    if ((method.getModifiers() & Modifier.STATIC)==0) {
      throw new RuntimeException("Method expected to be static.");
    }
    this.method=method;
  }

  public Object function(Object param) throws Impossible {
    Tuple args=(Tuple)param;

    Object[] paramArr=new Object[args.getArity()];
    for (int i=0;i<paramArr.length;i++) {
      paramArr[i]=args.getComponent(i);
    }

    try {
      return Executor.executeMethod(method,null,paramArr);
    }
    catch (Exception e) {
	if (e instanceof InvocationTargetException) {
	    ((InvocationTargetException)e).getTargetException().printStackTrace();
	}
      throw new Impossible("An exception occured during a method call.");
    }
  }

  /**
   * Serialization method, behaves like default writeObject
   * method. Stores the not-really-transient method field.
   **/
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException {
      out.defaultWriteObject();
      ReflectionSerializer.writeMethod(out, method);
  }

  /**
   * Deserialization method, behaves like default readObject
   * method. Restores the not-really-transient method field.
   **/
  private void readObject(java.io.ObjectInputStream in)
      throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      method = ReflectionSerializer.readMethod(in);
  }

}
