package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.ColorMap;
import de.renew.simulator.*;
import de.renew.event.*;

class PlaceInstanceFigure extends AbstractFigure
                          implements InstanceFigure,
				     PlaceEventListener,
				     FigureChangeListener {

  private PlaceFigure placeFigure;
  private PlaceInstance placeInstance;

  private boolean isMarked, expanded;
  private CPNInstanceDrawing drawing;

  public PlaceInstanceFigure(CPNInstanceDrawing drawing, PlaceFigure pf,
			     PlaceInstance pi) {
      placeInstance = pi;
      placeFigure = pf;
      pf.addFigureChangeListener(this);
      this.drawing = drawing;
      int ma = placeFigure.getMarkingAppearance();
      expanded = ma==PlaceFigure.EXPANDED;
      markingChanged(new PlaceEvent(placeInstance));
      if (ma != PlaceFigure.HIGHLIGHT) {
	  addTokenBagFigure();
      }
      placeInstance.addPlaceEventListener(this);
  }

  RenewMode getMode() {
      return drawing.getMode();
  }

  protected PlaceInstance getInstance() {
      return placeInstance;
  }

  synchronized public boolean isHighlighted() {
      return isMarked;
  }

  /** This event method is called whenever the marking of
   *  the corresponding PlaceInstance changes dramatically
   *  (i.e. by more than one token).
   */
  public void markingChanged(PlaceEvent pe) {
      if (pe.getPlaceInstance() != placeInstance) {
	  System.out.println("Wrong PlaceEvent received!");
	  throw new RuntimeException();
      }
      placeInstance.lock.lock();
      boolean newIsMarked = !placeInstance.isEmpty();
      placeInstance.lock.unlock();
      if (newIsMarked != isMarked) {
	  synchronized(this) {
	      isMarked = newIsMarked;
	  }
	  invalidate();
	  Figure hilight=placeFigure.getHighlightFigure();
	  if (hilight!=null) {
	      Rectangle area = hilight.displayBox();
	      area.grow(5,5);
	      DrawingChangeEvent dce = new DrawingChangeEvent(drawing,area);
	      drawing.drawingInvalidated(dce);
	  }
	  if (listener() != null)
	      listener().figureRequestUpdate(new FigureChangeEvent(this));
      }
  }

  /** This event occurs when a single token (contained in the
   *  event object) is put into the corresponding PlaceInstance.
   */
  public void tokenAdded(TokenEvent te) {
      markingChanged(new PlaceEvent(te.getPlaceInstance()));
  }

  /** This event occurs when a single token (contained in the
   *  event object) is removed from the corresponding PlaceInstance.
   */
  public void tokenRemoved(TokenEvent te) {
      tokenAdded(te);
  }

  /** This event occurs when a token (contained in the
   *  event object) is tested within the corresponding PlaceInstance.
   */
  public void tokenTested(TokenEvent te) {
      tokenAdded(te);
  }

  /** This event occurs when a token (contained in the
   *  event object) is untested within the corresponding PlaceInstance.
   */
  public void tokenUntested(TokenEvent te) {
      tokenAdded(te);
  }


    public void basicDisplayBox(Point origin, Point corner) {
        // do nothing, as the DisplayBox of the Place is used!
    }

    public Vector handles() {
        Vector handles = new Vector();
        handles.addElement(new NullHandle(this, RelativeLocator.northWest()));
        handles.addElement(new NullHandle(this, RelativeLocator.northEast()));
        handles.addElement(new NullHandle(this, RelativeLocator.southWest()));
        handles.addElement(new NullHandle(this, RelativeLocator.southEast()));
        return handles;
    }

    public Rectangle displayBox() {
        return placeFigure.displayBox();
    }

    protected void basicMoveBy(int x, int y) {
        // do nothing, as the DisplayBox of the Place is used!
    }

    public void draw(Graphics g) {
	// don't draw anything!
    }


    private TokenBagFigure addTokenBagFigure() {
      TokenBagFigure tbf=new TokenBagFigure(getMode(),this,
					    placeInstance,expanded);
      drawing.add(tbf);
      return tbf;
    }

  public boolean inspect(DrawingView view, boolean alternate) {
     if (alternate) {
       ((CPNApplication)view.editor()).openTokenBagDrawing(placeInstance);
       return true;
     } else {
	 if (getTokenBagFigure()==null) {
	     view.clearSelection();
	     view.addToSelection(addTokenBagFigure());
	     return true;
	 } else {
	     return super.inspect(view,alternate);
	 }
     }
  }

  public void setAttribute(String attribute, Object value) {
      if ("MarkingAppearance".equals(attribute)) {
	  int ma=((Integer)value).intValue();
	  TokenBagFigure etbf=getTokenBagFigure();
	  if (ma==PlaceFigure.HIGHLIGHT) {
	      if (etbf!=null) {
		  drawing.remove(etbf);
	      }
	  } else {
	      expanded = ma==PlaceFigure.EXPANDED;
	      if (etbf==null) {
		  addTokenBagFigure();
	      } else {
		  etbf.setExpanded(expanded);
	      }
	  }
      } else {
	  super.setAttribute(attribute,value);
      }
  }

  public void release() {
    if (placeInstance != null) {
      placeInstance.removePlaceEventListener(this);
      // OK: Release reference to place instance so that it
      // can be garbage collected.
      placeInstance=null;
      drawing=null;
      //etbf=null;
      //placeFigure=null;
    }
    placeFigure.removeFigureChangeListener(this);
    super.release();
  }

  private TokenBagFigure getTokenBagFigure() {
      FigureEnumeration childenum=children();
      if (childenum.hasMoreElements()) {
	  return (TokenBagFigure)childenum.nextElement();
      } else {
	  return null;
      }
  }

  public void figureInvalidated(FigureChangeEvent e) {
  }

  public void figureChanged(FigureChangeEvent e) {
      // If my place changes, I change!
      changed();
  }

  public void figureRemoved(FigureChangeEvent e) {
      // My place figure has been removed -- that's bad!!!
  }

  public void figureRequestRemove(FigureChangeEvent e) {}

  public void figureRequestUpdate(FigureChangeEvent e) {}


}
