package de.renew.gui;

import java.awt.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;
import de.renew.unify.Tuple;
import de.renew.util.TextToken;

public class TokenFigure extends SimpleCompositeFigure {

   Object token;
   private boolean expanded=true;

   public TokenFigure(RenewMode mode, Object token) {
       this.token = token;
       String text=null;
       if (token instanceof NetInstance) {
           NetInstance instance=(NetInstance)token;
	   CPNDrawing drawing=mode.getSimulation()
	       .getDrawing(instance.getNet().toString());
	   Figure iconFigure = null;
	   if (drawing != null)
	     iconFigure = drawing.getIconFigure();
	   if (iconFigure==null) {
	       text = instance.toString();
	   } else {
	       iconFigure=(Figure)iconFigure.clone();
	       updateID(iconFigure,instance);
	       add(iconFigure);
	       layout();
	       return;
	   }
       } else {
	   text=getText();
       }
       add(new TextFigure(text,true));
       layout();
   }

   private void updateID(Figure figure, NetInstance instance) {
       if (figure instanceof TextFigure) {
	   TextFigure textFig=(TextFigure)figure;
	   textFig.setReadOnly(true);
	   textFig.setText(CPNInstanceDrawing.expandMacro(textFig.getText(),
							  instance));
       } else if (figure instanceof CompositeFigure) {
	   // search recursively:
	   FigureEnumeration figenum=figure.figures();
	   while (figenum.hasMoreElements()) {
	       updateID(figenum.nextFigure(),instance);
	   }
       }
   }


   protected void layout() {
       // should contain exactly one figure (the token)
      AbstractFigure single=(AbstractFigure)figures().nextFigure();
      Rectangle box=single.displayBox();
      fDisplayBox.setSize(box.getSize());
      single.basicDisplayBox(fDisplayBox.getLocation(),
			     new Point(fDisplayBox.x+fDisplayBox.width,
				       fDisplayBox.y+fDisplayBox.height));
   }

   protected boolean needsLayout() {
      return false;
   }


   private String getText() {
      StringBuffer output = new StringBuffer();

      if (token==null) {
	output.append("null");
      } else {
	if (token instanceof String) {
	  output.append('"').append((String)token).append('"');
	} else if (token instanceof TextToken) {
	  output.append(((TextToken)token).toTokenText());
	} else {
	  output.append(token);
	}
	if (!expanded) {
	  String classname=token.getClass().getName();
	  if (classname.startsWith("java.lang."))
            classname=classname.substring(10);
	  StringBuffer shortOutput = new StringBuffer(classname);
	  shortOutput.append('@').append(token.hashCode());
	  if (shortOutput.length() < output.length()) {
            output=shortOutput;
	  }
	}
      }
      
      return output.toString();
   }

   public boolean inspect(DrawingView view, boolean alternate) {
     if (alternate) {
	 //       toggleExpanded();
	 //       return true;
        return false;
     } else {
       if (token instanceof NetInstance) {
         ((CPNApplication)view.editor()).openInstanceDrawing((NetInstance)token);
       } else if (token instanceof Window && ((Window)token).isShowing()) {
          ((Window)token).setVisible(true);
          ((Window)token).requestFocus();
       } else {
         return super.inspect(view,alternate);
       }
       return true;
     }
   }

/*
   public void toggleExpanded() {
      expanded=!expanded;
      layout();
   }
*/
}

