package de.renew.gui.fs;

import java.io.*;
import java.awt.*;
import java.util.Vector;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.TextFigure;
import CH.ifa.draw.util.ColorMap;
import CH.ifa.draw.standard.*;
import de.renew.formalism.fs.*;
import de.renew.shadow.*;
import de.renew.gui.*;

public class ConceptFigure extends TextFigure implements NodeFigure {

  private String conceptStr="", appropStr="";
  private static IsaConnection fgIsaPrototype=new IsaConnection();
  private static IsaConnection fgDisIsaPrototype=new IsaConnection(false);
  private static AssocConnection fgAssocPrototype=new AssocConnection();

  public ConceptFigure() {
    super();
    setAttribute("FrameColor", ColorMap.color("Black"));
    setAttribute("FillColor", ColorMap.color("White"));
  }

  public Vector handles() {
    Vector handles = super.handles();
    handles.addElement(new ConnectionHandle(this, RelativeLocator.north(),
					    fgDisIsaPrototype));
    handles.addElement(new ConnectionHandle(this, RelativeLocator.south(),
					    fgIsaPrototype));
    handles.addElement(new ConnectionHandle(this, RelativeLocator.east(),
					    fgAssocPrototype));
    handles.addElement(new ConnectionHandle(this, RelativeLocator.west(),
					    fgAssocPrototype));
    return handles;
  }

  /* hack for setting ConceptFigures to editable: */
  public void read(CH.ifa.draw.util.StorableInput dr) throws IOException {
    super.read(dr);
    setReadOnly(false);
    /* hack for getting rid of parent PartitionFigure: */
    setParent(null);
  }

   public void drawFrame(Graphics g) {
       super.drawFrame(g);
       if (!"".equals(appropStr)) {
         /* separate the first line (name of concept) from the features: */
        Rectangle r = displayBox();
        FontMetrics metrics = g.getFontMetrics(getFont());
        int y = r.y+metrics.getHeight();
        g.drawLine(r.x, y, r.x+r.width-1, y);
      }
    }


   public void setText(String text) {
      super.setText(text);
      // split first and remaining lines:
      int splitPos = text.indexOf('\n');
      if (splitPos<0) {
         appropStr = "";
         conceptStr = text;
      } else {
         appropStr = text.substring(splitPos+1).trim();
         conceptStr = text.substring(0,splitPos);
      }
      conceptStr = conceptStr.trim();
//System.out.println("Concept name: \""+conceptStr+"\", approp: \""+appropStr);
   }

  public Rectangle displayBox() {
    Rectangle box = super.displayBox();
    return new Rectangle(box.x-5, box.y, box.width+10, box.height);
  }

  private ShadowConcept shadow=null;

  /** Build a shadow in the given shadow net.
    *  This shadow is stored as well as returned.
    */
  public ShadowNetElement buildShadow(ShadowNet net) {
      shadow = new ShadowConcept(net,conceptStr,appropStr);
      shadow.context = this;
      // System.err.println("shadow for concept "+conceptStr+" created!");
      return shadow;
  }

  /** Get the associated shadow, if any.
   */
  public ShadowNetElement getShadow() {
    return shadow;
  }

  public String getName() {
      return conceptStr;
  }

  public void release() {
    super.release();
    if (shadow!=null)
      shadow.discard();
  }  

  public boolean canBeParent(ParentFigure figure) {
      //if (figure instanceof PartitionFigure)
      //  return super.canBeParent(figure);
    // only PartitionFigures can be parents of ConceptFigures!
    //return false;
    return figure==null;
  }

}
