package de.renew.gui.fs;

import CH.ifa.draw.figures.*;
import java.awt.*;
import collections.*;
import java.util.*;
import java.io.*;
import de.uni_hamburg.fs.*;
import de.renew.formalism.fs.*;
import de.renew.gui.*;
import CH.ifa.draw.util.StorableInput;

public class FSFigure extends CPNTextFigure {

  transient private Node fs;
  transient private TagMap tagmap;
  transient private FontMetrics metrics;
  private int lineh, d, ascent;
  transient private TypeSystem ts=null; // The TypeSystem which has been used for parsing

  public FSFigure(FeatureStructure fs) {
    super(INSCRIPTION);
    this.fs = fs.getRoot();
    tagmap = new TagMap(this.fs);
    super.setText(" ");
    setReadOnly(true);
    ts=TypeSystem.instance();
  }

  public FSFigure() {
    super(INSCRIPTION);
    super.setText("[]");
    setReadOnly(false);
  }

  public void setText(String src)  {
    super.setText(src);
    FSNetParser fsParser=new FSNetParser(new java.io.StringReader(src));
    ts=TypeSystem.instance();
    UpdatableMap tags=new HashedMap();
    EquivRelation er = new EquivRelation();
    fs=null;
    try {
      fs = fsParser.parseFS(tags,er,"",new Vector(),new Vector());
    } catch (de.renew.formalism.java.ParseException ex) {
	System.err.println("FS-Parse-Exception: "+ex);
	return;
    }
    try {
	er.extensionalize();
    } catch (UnificationFailure uff) {
	System.err.println("FS not extensionalizable!");
	fs = null;
	return;
    }
    fs = er.getUnificator(fs);
    tagmap = new TagMap(fs,er,tags);
    //System.out.println("FS: "+new FeatureStructure(fs));
  }

  private void drawBrackets(Graphics g, int d, Rectangle r) {
    int yu = r.y + r.height-1, xr = r.x + r.width-1;
    g.drawPolyline(new int[] {r.x+d,r.x,r.x,r.x+d},
                   new int[] {r.y, r.y, yu, yu}, 4);
    g.drawPolyline(new int[] {xr-d, xr, xr, xr-d},
                   new int[] {r.y, r.y, yu, yu}, 4);
  }

  private void drawAngles(Graphics g, int d, Rectangle r) {
    int yu = r.y + r.height-1,
      ym = r.y + r.height/2,
      xr = r.x + r.width-1;
    g.drawPolyline(new int[] {r.x+d,r.x,r.x+d},
                   new int[] {r.y, ym, yu}, 3);
    g.drawPolyline(new int[] {xr-d, xr, xr-d },
                   new int[] {r.y, ym, yu}, 3);
  }

  private Dimension fExtent=null;

  private void initMetrics() {
      metrics = Toolkit.getDefaultToolkit().getFontMetrics(getFont());
      lineh =  metrics.getHeight();
      d = metrics.stringWidth(" ");
      ascent = metrics.getAscent();
  }

  public Rectangle displayBox() {
    Rectangle box = super.displayBox();
    if (fs==null)
	return box;
    if (fExtent == null) {
	//tagmap = new TagMap(this.fs); why??? instead:
        tagmap.resetVisited();
        initMetrics();
        fExtent = drawFS(null,box.x,box.y,fs,Type.TOP);
    }
    return new Rectangle(box.x, box.y, fExtent.width, fExtent.height);
  }


  public void draw(Graphics g) {
    if (fs == null) {
	super.draw(g);
    } else {
      g.setColor((Color)getAttribute("TextColor"));
      g.setFont(getFont());
      Rectangle box = super.displayBox();
      tagmap.resetVisited();
      initMetrics();
      fExtent = drawFS(g,box.x,box.y,fs,Type.TOP);
    }
  }
  

  private Dimension drawList(Graphics g,
			     int x, int y, int maxh,
			     Node fs) {
    int w = 0;
    Type nodetype = fs.getType();
    if (ts.tNEList().subsumes(nodetype)) {
      Node head;
      if (fs.hasFeature(TypeSystem.HEAD_FEATURE))
	head = fs.delta(TypeSystem.HEAD_FEATURE);
      else
	head = new FSNode();
      Dimension headdim = drawFS(g,x,y,head,Type.TOP);
      w = headdim.width + d;
      maxh = Math.max(maxh,headdim.height);

      Node tail;
      if (fs.hasFeature(TypeSystem.TAIL_FEATURE)) {
	tail = fs.delta(TypeSystem.TAIL_FEATURE);
      } else {
	tail = new FSNode();
      }

      Dimension taildim;
      if (tagmap.getTag(tail) == 0
	  || ts.tEList().equals(tail.getType())) {
	  taildim = drawList(g,x+w,y,maxh,tail);
	  if (taildim.width==0)
	      w -= d;
      } else {
	  taildim = drawFS(g,x+w + 2*d,y,tail, ts.tList());
	  // add list seperator line:
	  maxh = Math.max(maxh,taildim.height);
	  if (g != null) {
	      int lx=x+w+d/2;
	      g.drawLine(lx,y, lx,y+maxh);
	  }
	  w += 2*d;
      }
      w += taildim.width;
      maxh = Math.max(maxh,taildim.height);
    } else if (!ts.tEList().subsumes(nodetype)) {
	Dimension restdim = drawFS(g,x + 2*d,y,fs,Type.TOP);
	w = restdim.width + 2*d;
	maxh = Math.max(maxh,restdim.height);
	if (g != null) {
	    g.drawLine(x,y,x,y+maxh);
	}
    }
    return new Dimension(w,maxh);
  }

  private Dimension drawFS(Graphics g,
			   int x, int y, Node fs,
			   Type type) {
    int h = lineh, maxw = d;
    boolean isList = false;
    GraphicTag gtag = new GraphicTag(fs);
    if (gtag.width > 0) {
	x += gtag.width + d; // space between tag and fs
    }
    int left = x+d;
    Type nodetype = fs.getType();
    Dimension extent = new Dimension(0,lineh);
    if (!tagmap.isVisited(fs) && !nodetype.getName().equals("_Dummy")) {
      tagmap.setVisited(fs);
      if (ts.tList().subsumes(nodetype)) {
	if (!nodetype.equals(ts.tList())) {
	  Dimension listdim = drawList(g,left+2*d,y,lineh,fs);
	  h = listdim.height;
	  //if (h == 0)
	  //  h = lineh;
	  maxw = listdim.width + 4*d;
	  isList = true;
	}
      } else {
	if (!nodetype.equals(type)) {
	  String typestr = nodetype.toString();
	  if (g != null)
	    g.drawString(typestr, left, y+ascent);
	  maxw = metrics.stringWidth(typestr);
	} else
	  h = 0;
	Enumeration features = fs.featureNames();
	if (features.hasMoreElements())
	  h += d;
	else
	  h = lineh;
	while (features.hasMoreElements()) {
	  Name featureName = (Name)features.nextElement();
	  if (featureName.toString().equals("_dummy"))
	      continue;
	  String feature = featureName.toString() + ":";
	  int indent = d + metrics.stringWidth(feature);
	  Dimension subdim = drawFS(g,left+indent,y+h,fs.delta(featureName),
				    nodetype.appropType(featureName));
	  if (g != null)
	    g.drawString(feature, left, y+h+(subdim.height-lineh)/2+ascent);
	  maxw = Math.max(maxw,indent + subdim.width);
	  h += subdim.height + d;
	}
      }
      if (maxw > d || isList || gtag.tag == 0) {
	  extent = new Dimension(maxw+2*d,h);
	  if (g != null) {
	      Rectangle r = new Rectangle(new Point(x,y),extent);
	      if (isList)
		  drawAngles(g,2*d,r);
	      else
		  drawBrackets(g,d,r);
	  }
      }
    }
    gtag.draw(g,x-d,y,extent);
    return extent;
  }


  class GraphicTag {
      int tag;
      int width = 0;
      String tagstr=null;

      GraphicTag(Node fs) {
	  tag = tagmap.getTag(fs);
	  if (tag != 0) {
	      tagstr = String.valueOf(tag);
	      width = d + metrics.stringWidth(tagstr);
	  }
      }

      void draw(Graphics g, int x, int y, Dimension extent) {
	  if (tag != 0) {
	      if (g != null) {
		  int tagy = y+(extent.height-lineh)/2;
		  Color penCol=g.getColor();
		  g.setColor(Color.white);
		  g.fillRect(x-width,tagy,width,lineh);
		  g.setColor(penCol);
		  g.drawString(tagstr,x-width+d/2,tagy+ascent);
		  g.drawRect(x-width,tagy,width-1,lineh-1);
	      }
	      if (extent.width != 0)
		  extent.width += d;
	      extent.width += width;
	  }
      }
  }

  public void read(StorableInput dr) throws IOException {
      if (dr.getVersion() <= 5)
	  // Hack for old FS-Figures:
	  super.readWithoutType(dr);
      else
	  super.read(dr);
  }

}
