package de.renew.remote;

import java.rmi.*;

/**
 * A transition instance accessor allows to view and modify the
 * state of a transition instance.
 * <p>
 * This is a remote interface that allows an implementation
 * that can be used via RMI.
 * </p>
 *
 * TransitionInstanceAccessor.java
 * Created: Mon Jul 10  2000
 * @author Michael Duvigneau
 */

public interface TransitionInstanceAccessor extends RemoteEventProducer, Accessor {

  /**
   * Get the ID of the transition instance that is being accessed with
   * this object. The ID of a transition instance is the same as the
   * ID of the transition the instance is based on.
   *
   * @return the ID of the transition instance
   * @exception java.rmi.RemoteException if an RMI failure occured.
   */
  public int getID() throws RemoteException;

  /**
   * Get a transition accessor that corresponds to this accessor.
   *
   * @return the transition accessor
   * @exception java.rmi.RemoteException if an RMI failure occured.
   */
  public TransitionAccessor getTransition() throws RemoteException;

  /**
   * Get a net instance accessor that corresponds to this accessor.
   *
   * @return the net instance accessor
   * @exception java.rmi.RemoteException if an RMI failure occured.
   */
  public NetInstanceAccessor getNetInstance() throws RemoteException;
  
}
