package de.renew.shadow;

public interface ShadowCompiler extends java.io.Serializable {
  // A shadow compiler must be serialisable.
  // Usually this is no problem.

 /** Compile a net system and return a lookup that knows
  *  about the compiled nets.
  */
  public ShadowLookup compile(ShadowNetSystem netSystem)
    throws SyntaxException;
    
 /** Return an array of names of special checks that are not
  *  required during compilation, but can help to find
  *  common mistakes in the models. getLintNames().length
  *  gives the number of possible checks.
  */
  public String[] getLintNames();  

 /** Perform one special check.
  *  @param lintNr The number of the check, starting with 0.
  *  @param netSystem The NetSystem to be checked.
  *  @param selectedNet May contain one of the ShadowNets of 
  *      netSystem that is to be treated specially.
  */
  public void lint(int lintNr, ShadowNetSystem netSystem,
		   ShadowNet selectedNet, ShadowNet lastSelectedNet)
    throws SyntaxException;
    
 /** Perform a syntax check for a string. The string is
  *  parsed, if possible, in the context of a given
  *  shadow net, so that declarations can be handled.
  * 
  *  The method returns a string that denotes the type of
  *  inscription that was parsed. If the type could not be uniquely
  *  determined, null is returned.
  */
  public String checkDeclarationNode(String inscr, boolean special)
    throws SyntaxException;
  public String checkArcInscription(String inscr, boolean special,
				    ShadowNet shadowNet)
    throws SyntaxException;
  public String checkTransitionInscription(String inscr, boolean special,
					   ShadowNet shadowNet)
    throws SyntaxException;
  public String checkPlaceInscription(String inscr, boolean special,
				      ShadowNet shadowNet)
    throws SyntaxException; 
}
