package de.renew.simulator;

import de.renew.util.SingletonEnumeration;
import de.renew.expression.*;
import collections.*;

public class Arc implements TransitionInscription {
  // The arc types that are prefixed with a star require a time expression.
  public static final int in=-1;       // * input arc
  public static final int test=0;      // test arc
  public static final int out=1;       // * output arc
  public static final int both=2;      // * double arc that reserves the token
                                       // and applies the time to the input
                                       // token
  public static final int fastBoth=3;  // * double arc that releases the token
                                       // early during the firing
  public static final int fastTest=4;  // test arc that releases the token
                                       // early during the firing
  public static final int inhibitor=5; // inhibitor arc
  public static final int bothOT=6;    // * double arc that reserves the token
                                       // and applies the time to the output
                                       // token
  Place place;
  int arcType;
  Expression tokenExpr;
  Expression timeExpr;
  boolean trace;

  public Arc(Place place, int arcType,
	     Expression tokenExpr, Expression timeExpr)
  {
    if (arcType<in || arcType>bothOT) {
      throw new RuntimeException("Bad arc type: "+arcType+".");
    }
    this.place=place;
    this.arcType=arcType;
    this.tokenExpr=tokenExpr;
    this.timeExpr=timeExpr;
    trace=true;
  }

  public void setTrace(boolean trace) {
    this.trace=trace;
  }

  public boolean getTrace() {
    return trace;
  }

  public boolean isTestArc() {
    return arcType==test || arcType==fastTest;
  }

  public boolean isUntimedArc() {
    return isTestArc() || arcType==inhibitor;
  }

  public CollectionEnumeration makeOccurrences(VariableMapper mapper,
    NetInstance netInstance, Searcher searcher)
  {
    return new SingletonEnumeration(new ArcOccurrence(this,
      mapper,netInstance));
  }
}

