package de.renew.simulator;

import de.renew.expression.*;
import collections.*;
import de.renew.util.*;
import de.renew.unify.*;

// A conditional occurrence is its own binder. This simplifies
// the code a bit.

class ConditionalOccurrence implements InscriptionOccurrence, Binder {
  ConditionalInscription conditionalInscription;

  VariableMapper mapper;
  NetInstance netInstance;

  Variable conditionVariable;

  private UpdatableSeq secondaryOccurrences;
  private boolean wantToOccur;
  
  ConditionalOccurrence(ConditionalInscription conditionalInscription,
    VariableMapper mapper, NetInstance netInstance)
  {
    this.conditionalInscription=conditionalInscription;
    this.mapper=mapper;
    this.netInstance=netInstance;
  }
  
  public CollectionEnumeration makeBinders(Searcher searcher)
    throws Impossible
  {
    conditionVariable=new Variable(
      conditionalInscription.conditionExpression.startEvaluation(mapper,
	searcher.recorder,searcher.calcChecker),
	searcher.recorder);

    // I am my own binder.
    return new SingletonEnumeration(this);
  }

  public int bindingBadness(Searcher searcher) {
    if (Unify.isBound(conditionVariable)) {
      return 1;
    } else {
      return BindingBadness.max;
    }
  }

  public void bind(Searcher searcher) {
    Object obj=conditionVariable.getValue();
    if (!(obj instanceof Value)) {
      // Expression is badly bound.
      return;
    }

    Object value=((Value)obj).value;
    if (!(obj instanceof Boolean)) {
      // Expression is badly bound.
      return;
    }

    UpdatableSeq binders=new LinkedList();

    int checkpoint=searcher.recorder.checkpoint();

    try {       
      boolean wantToOccur=((Boolean)((Value)obj).value).booleanValue();
      
      secondaryOccurrences=new LinkedList();
      if (wantToOccur) {
	secondaryOccurrences.prependElements(
	  conditionalInscription.inscription.makeOccurrences(mapper,
	    netInstance,searcher));
	
	// Create all binders.
	CollectionEnumeration enum=secondaryOccurrences.elements();
	while (enum.hasMoreElements()) {
	  InscriptionOccurrence occurrence=
	    (InscriptionOccurrence)enum.nextElement();
	  binders.prependElements(occurrence.makeBinders(searcher));
	}

	// After this statement, no further exceptions should be thrown.
	// Add binders to searcher.
	searcher.addBinders(binders.elements());
      }
      
      searcher.removeBinder(this);
      searcher.search();
      searcher.addBinder(this);
      
      if (wantToOccur) {
	searcher.removeBinders(binders.elements());
	secondaryOccurrences=null;
      }
    }      
    catch (Impossible e) {
      // This is expected. The secondary occurrence could
      // not create its binders.
    }
    
    searcher.recorder.restore(checkpoint);
  }

  public CollectionEnumeration makeExecutables(Copier copier,
    VariableMapper copiedMapper)
  {
    UpdatableSeq executables=new LinkedList();

    if (wantToOccur) {
      // Create all executables.
      CollectionEnumeration enum=secondaryOccurrences.elements();
      while (enum.hasMoreElements()) {
	InscriptionOccurrence occurrence=
	  (InscriptionOccurrence)enum.nextElement();
	
	executables.prependElements(occurrence.makeExecutables(copier,
	  copiedMapper));
      }
    }
    
    return executables.elements();
  }
}
