package de.renew.simulator;

import de.renew.expression.*;
import collections.*;
import de.renew.unify.*;
import de.renew.util.*;

class FlexibleArcOccurrence implements InscriptionOccurrence {
  PlaceInstance placeInstance;
  VariableMapper mapper;
  Variable tokenVar;
  FlexibleArc arc;

  FlexibleArcOccurrence(FlexibleArc arc, VariableMapper mapper,
    NetInstance netInstance)
  {
    this.arc=arc;
    this.mapper=mapper;
    placeInstance=netInstance.getInstance(arc.place);
  }

  public CollectionEnumeration makeBinders(Searcher searcher)
    throws Impossible 
  {
    tokenVar=new Variable(
      arc.expression.startEvaluation(mapper,searcher.recorder,
        searcher.calcChecker),
      searcher.recorder);
    if (arc.arcType==FlexibleArc.out) {
      searcher.calcChecker.addLateVariable(tokenVar,
	searcher.recorder);
      return new EmptyEnumeration();
    } else {
      return new SingletonEnumeration(new FlexibleArcBinder(
	tokenVar,searcher.reserver,placeInstance,arc));
    }
  }

  public CollectionEnumeration makeExecutables(Copier copier,
    VariableMapper copiedMapper)
  {
    switch (arc.arcType) {
    case FlexibleArc.out:
      Variable copiedTokenVar=(Variable)copier.copy(tokenVar);
      return new SingletonEnumeration(
        new FlexibleOutArcExecutable(placeInstance,copiedTokenVar,arc));
    case FlexibleArc.in:
    case FlexibleArc.fastBoth:
      return new SingletonEnumeration(
        new FlexibleInArcExecutable(placeInstance,tokenVar.getValue(),arc));
    default:
      throw new RuntimeException("Bad arc type.");
    }
  }
}
