package de.renew.simulator;

/**
 * An ID factory is responsible for creating unique ID strings.
 * Depending on the requirements of the application, a factory
 * may generate IDs that are unique to this process, to this application,
 * or even globally unique.
 */

public interface IDFactory {
  /**
   * Create another ID.
   *
   * @return a unique string
   */
  public String createID();

  /**
   * Create another ID for a net instance.
   *
   * @param net the net that defines the net instance to be created.
   * @return a unique string
   */
  public String createID(Net net);
}
