package de.renew.simulator;

import collections.*;
import de.renew.unify.*;

class InputArcExecutable implements EarlyExecutable {
  PlaceInstance placeInstance;
  Object token;
  double delay;
  boolean trace;
  boolean releaseImmediately;

  // This field will store the time stamp of the removed token
  // after a successful removal.
  double removeTime;

  InputArcExecutable(PlaceInstance placeInstance, Object token,
    double delay, boolean releaseImmediately, boolean trace)
  {
    this.placeInstance=placeInstance;
    this.token=token;
    this.delay=delay;
    this.trace=trace;
    // If releaseImmediately is true, the arc behaves like
    // a double arc that puts the token back as soon as possible.
    this.releaseImmediately=releaseImmediately;
  }

  public long lockPriority() {
    return placeInstance.lockOrder;
  }

  public int phase() {
    return INPUT;
  }

  public void lock() {
    placeInstance.lock.lock();
  }

  public void verify() throws Impossible {
    try {
      removeTime=placeInstance.removeToken(token,delay);
    }
    catch (Impossible e) {
      // No reservation actually took place.
      throw e;
    }
  }

  static String makeTraceString(String s1, Object token, String s2,
				PlaceInstance placeInstance) 
  {
    StringBuffer buf=new StringBuffer("    ");
    buf.append(s1);
    buf.append(" ");
    buf.append(token);
    buf.append(" ");
    buf.append(s2);
    buf.append(" ");
    buf.append(placeInstance);

    return buf.toString();
  }
    
  static void traceInArc(Tracer tracer, boolean checking,
			 Object token, PlaceInstance placeInstance) 
  {
    if (checking) {
      tracer.trace(makeTraceString("Checking",token,
				   "in",placeInstance));
    } else {
      tracer.trace(makeTraceString("Removing",token,
				   "from",placeInstance));
    }
  }

  public void execute(Tracer tracer) {
    if (releaseImmediately) {
      if (trace) {
	  traceInArc(tracer,true,token,placeInstance);
      }     
      // It would be conceivable to put the token back at
      // a different time.
      placeInstance.insertToken(token,SearchQueue.getTime());
    } else {
      if (trace) {
	  traceInArc(tracer,false,token,placeInstance);
      }     
    }
  }

  public void rollback() {
    // We have to undo the previous removal. We cannot do this
    // without notifying the observers and listeners, because
    // the removal was not done silently.
    placeInstance.insertToken(token,removeTime);
  }

  public void unlock() {
    placeInstance.lock.unlock();
  }
}
