package de.renew.simulator;

import collections.CollectionEnumeration;
import de.renew.unify.Copier;
import de.renew.unify.Impossible;
import de.renew.expression.VariableMapper;

public interface InscriptionOccurrence {
  /**
   * Prepare the binders that allow this inscription to act.
   * It is guaranteed that this method will be invoked only once.
   * <p>
   * If this method throws a Impossible, then there
   * cannot be a suitable binding. Possible bindings made using
   * the state recorder are not rolled back, this is left to 
   * the caller.
   * </p>
   *
   * @see Binder
   **/
  CollectionEnumeration makeBinders(Searcher searcher)
    throws Impossible;

  /**
   * Provide an enumeration of executable objects that will
   * perform the neccessary actions for this inscription occurrence.
   * At this point, such executables must be available,
   * i.e. all binders have agreed on the current binding.
   *
   * @see Executable
   **/
  CollectionEnumeration makeExecutables(Copier copier,
    VariableMapper copiedMapper);
}
