package de.renew.simulator;

import collections.CollectionEnumeration;
import de.renew.unify.Copier;
import de.renew.expression.VariableMapper;

/**
 * I represent a something that occurs, e.g. a transition
 * occurrence. Upon successful search, I allow executables
 * to be extracted.
 *
 * @author Olaf Kummer
 **/

public interface Occurrence {
  /**
   * I will provide executables that
   * perform the required actions according to the
   * current bindings of variables. This includes, among other
   * actions, token moves and tests.
   *
   * All binders must have been processed before it is allowed
   * to call this method.
   *
   * @param copier
   *   a copier that I can use to rescue the current values of
   *   variables from backtracking
   * @return an enumeration of executables
   *
   * @see de.renew.simulator.Executable
   **/
  public CollectionEnumeration makeExecutables(Copier copier);

  /**
   * This provides access to my variable mapper.
   *
   * @return a variable mapper
   **/
  public VariableMapper getMapper();
}
