package de.renew.simulator;

/**
 * This ID factory creates unique IDs by counting from 0 upwards.
 * This will generate IDs that are unique within a JVM.
 *
 * This is for non-persistent applications only.
 */

public class TrivialIDFactory implements IDFactory {
  private long count=0;

  /**
   * Create another ID. Each call will return the string
   * representation of a number that advances by one on
   * each call.
   *
   * @return a unique string
   */
  public String createID() {
    return String.valueOf(count++);
  }

  /**
   * Create another ID for a net instance by delegating to the net.
   *
   * @param net the net that defines the net instance to be created.
   * @return a unique string
   */
  public String createID(Net net) {
    return String.valueOf(net.makeNetNumber());
  }
}
