package de.renew.util;

import java.io.ObjectInput;
import java.io.IOException;

public class ClassSource {
  // This class is completely static.
  private ClassSource() {}

  private static ClassLoader loader=null;
  private static ReloadableDeserializer deserializer=null;

  public static synchronized void setClassLoader(ClassLoader newLoader) {
    loader=newLoader;
    try {
      deserializer=(ReloadableDeserializer)
	classForName("de.renew.util.ReloadableDeserializerImpl").newInstance();
    }
    catch (Exception e) {
      System.err.println(e);
      throw new RuntimeException("Cannot generate deserializer.");
    }
  }

  public static synchronized Class classForName(String name)
    throws ClassNotFoundException
  {
    if (loader==null) {
      //System.err.println("Loading class "+name);
      return Class.forName(name); // No reload requested.
    } else {
      // Use class loader to facilitate possible reload.
      //System.err.println("Reloading class "+name+" with loader "+loader);
      return loader.loadClass(name);
    }
  }

  // Make sure to establish a reloaded object on the call stack
  // so that deserialization uses our class loader and not the
  // default class loader.
  public static Object readObject(ObjectInput input)
    throws ClassNotFoundException, IOException
  {
    if (deserializer==null)
      return input.readObject();
    else
      return deserializer.readObject(input);
  }
}

interface ReloadableDeserializer {
  public Object readObject(ObjectInput input)
    throws ClassNotFoundException, IOException;
}

class ReloadableDeserializerImpl implements ReloadableDeserializer {
  public Object readObject(ObjectInput input)
    throws ClassNotFoundException, IOException
  {
    return input.readObject();
  }  
}
