package de.renew.util;

import collections.CollectionEnumeration;
import java.io.Serializable;

// I convert nulls to and from an internal representation,
// so that the collection classes do not need to treat
// nulls explicitly.

// Ideally there is only one instance of Null: NULL. 
// Unfortunately on deserialization several instances
// of Null can be created. The equals() and hashCode()
// methods ensure compatibility between these instances
// and the one true NULL.
// Consequence: identity (==) must not be used to check
//              for identity, use equals() instead.
//              (... instanceof Null should work, too)

public class Null implements Serializable {
  private final static Object NULL=new Null();

  // Convert a single object.
  public final static Object convert(Object obj) {
    if (obj==null) {
      return NULL;
    } else if (obj instanceof Null) {
      return null;
    }
    return obj;
  }

  // Convert an enumeration. I simply put a wrapper object around
  // the original enumeration.
  public final static CollectionEnumeration convertEnumeration(
    CollectionEnumeration enum)
  {
    return new NullEnumeration(enum);
  }
  
  public String toString() {
    return "null";
  }

  public boolean equals(Object obj) {
    return obj instanceof Null;
  }

  public int hashCode() {
    return 0;
  }
}
