package de.uni_hamburg.fs;

import java.util.*;

public interface Concept {

  /** Return the name of this Concept. */
  public String getName();

  /** Return whether this Concept is extensional. */
  public boolean isExtensional();

  /** Return whether this Concept restricts which features are allowed.
   *  A non-restricted Concept is appropriate for every feature.
   */
  public boolean isRestricted();

  /** Return whether the feature <feature> is appropriate in this Concept. 
   */
  public boolean isApprop(Name featureName);

  /** Return the required ConceptSet for the Value under the given feature. */
  public ConceptSet appropConcepts(Name featureName)
                    throws NoSuchElementException;

  /** Return the required Type for the Value under the given feature. */
  public Type appropType(Name featureName)
              throws NoSuchElementException;

  /** Return an ordered Enumeration of all appropriate features.
   *  A non-restrictive Concept returns only an Enumeration of the
   *  FeatureNames for which it defines restricted Types (since it
   *  cannot return an Enumeration of all possible FeatureNames).
   */
  public Enumeration appropFeatureNames();

  /** Return whether this Concept is-a <that> Concept.
    * In other words, return wheter this Concept is more special than <that>
    * Concept.
    */
  public boolean isa(Concept that);

  /** Return whether this Concept is-not-a <that> Concept.
    * In other words, return wheter this Concept is incompatible with <that>
    * Concept.
    */
  public boolean isNotA(Concept that);

  public ConceptSet unify(Concept that) throws UnificationFailure;

  public ConceptEnumeration extensionalSuperconcepts();

}

