package de.uni_hamburg.fs;

import collections.*;
import java.util.*;

public class ConceptSet {

  public static final ConceptSet EMPTY = new ConceptSet();

  private UpdatableSet concepts = new HashedSet();

  public ConceptSet() { }

  public ConceptSet(Concept con) {
      concepts.include(con);
  }

  public ConceptEnumeration elements() {
    return new ConceptEnumeration(concepts.elements());
  }

  public void addConcept(Concept newConcept) {
      concepts.include(newConcept);
  }

  public void unite(ConceptSet that) {
      concepts.includeElements(that.elements());
  }

  public void joinConcept(Concept newConcept) throws UnificationFailure {
      UpdatableSet newConcepts = new HashedSet();
      ConceptEnumeration conceptEnum = elements();
      while (conceptEnum.hasMoreElements()) {
        Concept concept = conceptEnum.nextConcept();
        if (concept.isa(newConcept))
           return; // already something more special there
        if (concept.isNotA(newConcept))
           throw new UnificationFailure(); // incompatible!
        // only include concepts which are not more general than the new:
        if (!newConcept.isa(concept)) {
           newConcepts.include(concept); 
        }
      }
      newConcepts.include(newConcept);
      concepts = newConcepts;
  }

  public void join() throws UnificationFailure {
      ConceptSet newConcepts = new ConceptSet();
      newConcepts.joinConcepts(elements());
      concepts=newConcepts.concepts;
  }

  public void joinConcepts(ConceptEnumeration conceptEnum)
              throws UnificationFailure {
      while (conceptEnum.hasMoreElements()) {
	joinConcept(conceptEnum.nextConcept());
      }
  }

  public void meetConcept(Concept newConcept) {
      UpdatableSet newConcepts = new HashedSet();
      ConceptEnumeration conceptEnum = elements();
      while (conceptEnum.hasMoreElements()) {
        Concept concept = conceptEnum.nextConcept();
        if (newConcept.isa(concept))
           return; // already something more general there
        // only include concepts which are not more special than the new:
        if (!concept.isa(newConcept)) {
           newConcepts.include(concept); 
        }
      }
      newConcepts.include(newConcept);
      concepts = newConcepts;
  }

  public void meet() {
      ConceptSet newConcepts = new ConceptSet();
      ConceptEnumeration conceptEnum = elements();
      while (conceptEnum.hasMoreElements()) {
        Concept concept = conceptEnum.nextConcept();
	newConcepts.meetConcept(concept);
      }
      concepts = newConcepts.concepts;
  }

  public boolean equals(Object that) {
      if (that instanceof ConceptSet) {
	  return ((ConceptSet)that).concepts.sameStructure(concepts);
      }
      return false;
  }

  /** Return a String representation of this ConceptSet. */
  public String toString() {
    StringBuffer output = new StringBuffer('{');
    ConceptEnumeration conceptEnum = new ConceptEnumeration(concepts.elements());
    for (int i = 1; conceptEnum.hasMoreElements(); ++i) {
      if (i > 1)
	output.append(',');
      Concept concept = conceptEnum.nextConcept();
      output.append(concept.getName());
    }
    output.append('}');
    return output.toString();
  }


}
