package de.uni_hamburg.fs;

import collections.*;
import java.util.*;

public class JavaConcept extends SimpleConcept {

  private Class javaClass;

  /** Construct a new wrapper for the given Java Type. */
  JavaConcept(Class javaClass) {
    this.javaClass = javaClass;
  }

  /** Return the name of this JavaConcept. */
  public String getName() {
    return javaClass.getName();
  }

 /** Return the Java Type that is wrapped by this JavaConcept.
   */
  public Class getJavaClass() {
     return javaClass;
  }

  /** Return whether this JavaConcept is-a <that> Concept.
    * In other words, return wheter this Concept is more special than <that>
    * Concept.
    */
  public boolean isa(Concept that) {
    if (that instanceof JavaConcept) {
        Class thatJavaClass = ((JavaConcept)that).getJavaClass();
        return thatJavaClass.isAssignableFrom(javaClass);
    }
    return false;
  }

  /** Return whether this JavaConcept is-not-a "that" Concept.
    * In other words, return wheter this JavaConcept is incompatible with 
    *  "that" JavaConcept. This is the case if
    * <ol><i>the other Concept is not a JavaConcept,
    *        <i>both JavaConcepts are classes which are not in a subclass 
    *              relation, or</i>
    *        <i> the Java classes/interfaces define methods in an 
    *               incompatible way.</i></ol>
    */
   public boolean isNotA(Concept that) {
    if (this.isa(that) || that.isa(this))
       return false;
    if (that instanceof JavaObjectFS) {
       return true; // the only chance was: that.isa(this)
    }
    if (javaClass.isInterface())
       return false; // method-inconsistencies are not checked yet!
    if (that instanceof JavaConcept) {
        Class thatJavaClass = ((JavaConcept)that).getJavaClass();
        if (thatJavaClass.isInterface())
           return false; // method-inconsistencies are not checked yet!
    }
    return true;
  }

}
