package de.uni_hamburg.fs;

import collections.*;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import de.renew.util.Value;

public class JavaObjectFS extends SimpleConcept implements Node {

  private Object javaObject;
  private JavaConcept concept;
  private Type type;

  /** Construct a new wrapper for the given Java Object. */
  public JavaObjectFS(Object javaObject) {
    this.javaObject = javaObject;
    Class clazz;
    if (javaObject instanceof Value) {
        clazz=de.renew.util.Types.typify(((Value)javaObject).value.getClass());
    } else {
        clazz=javaObject.getClass();
    }
    concept = TypeSystem.instance().getJavaConcept(clazz);
    type = new ConjunctiveType(this);
  }

  /** Return the Java Object that is wrapped by this JavaObjectFS.
   */
  public Object getJavaObject() {
     return javaObject;
  }

  public int hashCode() { return javaObject.hashCode(); }

 public boolean equals(Object that) {
     if (that instanceof JavaObjectFS) {
        return javaObject.equals(((JavaObjectFS)that).javaObject);
     } else {
        return javaObject.equals(that);
     } /* endif */
  }

 /* Implementation of Node methods: */

  /** Return an Enumeration of the codes of all Features of this Feature Structure. */
  public Enumeration featureNames() {
     return SimpleConcept.emptySet.elements();  // A JavaObjectFS has no features.
  }

  /** Returns whether this Feature Structure has the Feature given by its code. */
  public boolean hasFeature(Name featureName) {
     return false;  // A JavaObjectFS has no features.
  }

  /** Returns the Feature Structure at the Feature given by its code. */
  public Node delta(Name featureName) throws NoSuchElementException {
     throw new NoSuchElementException(); // A JavaObjectFS has no features.
  }

  public Node at(String path) throws NoSuchElementException {
     throw new NoSuchElementException(); // A JavaObjectFS has no features.
  }

  public void updateFeature(Name featureName, Node newValue) { }

  /* Implementation of Concept methods: */

  /** Return the name of this Concept. */
  public String getName() {
     if (javaObject instanceof String)
        return '"' + javaObject.toString() + '"';
     else 
        return javaObject.toString();
  }

  /** Return whether this Concept is-a <that> Concept.
    * In other words, return wheter this Concept is more special than <that>
    * Concept.
    */
  public boolean isa(Concept that) {
    if (that instanceof JavaObjectFS) {
       Object thatJavaObject = ((JavaObjectFS)that).getJavaObject();
       return javaObject.equals(thatJavaObject);
    } else if (that instanceof Bound) {
       return ((Bound)that).subsumesObject(javaObject);
    } else
        return concept.isa(that);
  }

  /** Return whether this Concept is-not-a <that> Concept.
    * In other words, return wheter this Concept is incompatible with <that>
    * Concept.
    */
  public boolean isNotA(Concept that) {
    return !isa(that);
  }

  /** Get the Type of the Feature Structure. */
  public Type getType() {
     return type;
  }

  public Node duplicate() {
     return this; // A JavaObjectFS is extensional and thus need not be copied!
  }

}
