package de.uni_hamburg.fs;

import collections.*;
import java.util.*;

/** The abstract class Node defines all methods that are needed
  * to construct, access and unify Feature Structures Nodes.
  * A Node has a Type and a number of Features that as a value
  * contain another Node of a Type appropriate for that Feature.
  */
public interface Node {

  /** Get the Type of the Node. */
  public Type getType();

  /** Return an Enumeration of the codes of all Features of this Node. */
  public Enumeration featureNames();

  /** Returns whether this Node has the Feature given by its code. */
  public boolean hasFeature(Name featureName);

  /** Returns the Node at the Feature given by its code. */
  public Node delta(Name featureName) throws NoSuchElementException;

  /** Returns the Node at the given path.
    *  A path is a sequence of Features (meant to be starting at the root node).
    *  The syntax of a path is feature_1:feature_2:...feature_n.
    *  The empty String returns the Node itself.
    *  The exception is thrown if at any point, the feature given
    *  by the path does not exist in the current Node.
    */
  public Node at(String path) throws NoSuchElementException;

  public Node duplicate();

  public void updateFeature(Name featureName, Node newValue);

}

