package de.uni_hamburg.fs;

import java.util.*;

public class OrderedTable {
  // OK: These three used to be final, but that triggers the
  // all too well-known final bug.
  private Hashtable table;
  private Vector keys;
  private Vector orderSource;

  public OrderedTable() {
      this(null);
  }

  public OrderedTable(Enumeration orderEnum) {
      table=new Hashtable();
      keys=new Vector();
      if (orderEnum==null) {
	  orderSource=null;
      } else {
	  orderSource=new Vector();
	  while (orderEnum.hasMoreElements()) {
	      orderSource.addElement(orderEnum.nextElement());
	  }
      }
  }

  private OrderedTable(Hashtable table, Vector keys, Vector orderSource) {
      this.table=(Hashtable)table.clone();
      this.keys=(Vector)keys.clone();
      this.orderSource=orderSource; // not modified
  }

  public void putAt(Object key, Object value) {
      if (!keys.contains(key)) {
	  int index;
	  if (orderSource==null) {
	      index = keys.size();
	  } else {
	      // find insert position according to orderSource:
	      index = 0;
              for (int i=0; i<orderSource.size(); ++i) {
		  Object currkey=orderSource.elementAt(i);
		  if (key.equals(currkey)) {
		      break;
		  }
		  if (index<keys.size() 
		      && keys.elementAt(index).equals(currkey)) {
		      ++index;
		  }
	      }
	  }
	  keys.insertElementAt(key,index);
      }
      table.put(key,value);
  }

  public void insertAt(Object key, int index, Object value) {
      keys.removeElement(key);
      keys.insertElementAt(key,index);
      table.put(key,value);
  }

  public void removeAt(Object key) {
      keys.removeElement(key);
      table.remove(key);
  }

  public boolean includesKey(Object key) {
      return keys.contains(key);
  }

  public int size() {
      return keys.size();
  }

  public Enumeration keys() {
      return keys.elements();
  }

  public Object at(Object key) {
      return table.get(key);
  }

  public Object clone() {
      return new OrderedTable(table,keys,orderSource);
  }
}

