package de.uni_hamburg.fs;

import collections.*;
import java.util.*;

public class PrettyPrinter {

  private TagMap map;

  public static void println(FeatureStructure fs) {
      System.out.println(toString(fs));
  }

  public static String toString(FeatureStructure fs) {
      Node node=fs.getRoot();
      return "\n" + new PrettyPrinter(node).toString(node,"\n ",Type.TOP)
	   + "\n";
  }

  private PrettyPrinter(Node node) {
      map = new TagMap(node);
  }

  private String toString(Node thiz, String indent, Type defaultType) {
    StringBuffer output = new StringBuffer();
//output.append('*').append(seqNo); indent += FeatureStructure.indent(output.length());
    int tag = map.getTag(thiz);
    if (tag != 0) {
      output = output.append("#").append(Math.abs(tag));
      indent += indent(output.length());
    }
    if (!map.isVisited(thiz)) {
      map.setVisited(thiz);
      output.append(untaggedToString(thiz,indent,defaultType));
    }
    return output.toString();
  }

  private String listToString(Node thiz, String indent) {
    StringBuffer output = new StringBuffer();
    if (TypeSystem.neList().subsumes(thiz.getType())) {
	  if (thiz.hasFeature(TypeSystem.HEAD_FEATURE))
	      output = output.append
		  (toString(thiz.delta
			    (TypeSystem.HEAD_FEATURE),indent+" ",Type.TOP))
		.append(indent);
	  else
	    output = output.append("(no head feature!)");
	  if (thiz.hasFeature(TypeSystem.TAIL_FEATURE)) {
	    Node tailfs = thiz.delta(TypeSystem.TAIL_FEATURE);
	    int tag = map.getTag(tailfs);
	    if (tag != 0 && !TypeSystem.eList().equals(tailfs.getType())) {
	      output = output.append("| ")
		.append(toString(tailfs,indent+"   ",TypeSystem.list()));
	    } else {
	      output = output.append(listToString(tailfs,indent));
	    }
	  } else
	    output = output.append("(no tail feature!)");
    } else if (!TypeSystem.eList().equals(thiz.getType())) {
      output.append("| ").append(toString(thiz,indent+"   ",Type.TOP));
    }
    return output.toString();
  }

  private String innerToString(Node thiz, String indent, Type defaultType) {
        StringBuffer output = new StringBuffer();
	String whitespace = "";
	if (!thiz.getType().equals(defaultType)) {
	  output = output.append(thiz.getType().getName());
	  whitespace = indent;
	}
	Enumeration features = thiz.featureNames();
	while (features.hasMoreElements()) {
	  Name featureName = (Name)features.nextElement();
	  String feature = featureName.toString();
	  output = output.append(whitespace).append(feature).append(": ")
	    .append(toString(thiz.delta(featureName),
			     indent+indent(3+feature.length()),
			     thiz.getType().appropType(featureName)));
	  whitespace = indent;
	}
	return output.toString();
  }

  public String untaggedToString(Node thiz, String indent,
				 Type defaultType) {
     StringBuffer output = new StringBuffer();
     if (thiz instanceof JavaObjectFS) {
	 output.append('(').append(((JavaObjectFS)thiz).getName()).append(')');
     } else {
	 if (TypeSystem.neList().subsumes(thiz.getType())
	     || TypeSystem.eList().equals(thiz.getType())) {
	     output = output.append("<").append(listToString(thiz,indent))
		 .append(">");
	 } else {
	     output.append("[").append(innerToString(thiz,indent,defaultType))
		 .append("]");
	 }
     }
     return output.toString();
  }

  public final static String indent(int depth) {
    StringBuffer space = new StringBuffer(depth+1);
    for (int i = 0; i < depth; ++i)
      space.append(' ');
    return space.toString();
  }


}
