package de.uni_hamburg.fs;

import collections.*;
import java.util.NoSuchElementException;
import java.util.Enumeration;

public abstract class SimpleConcept implements Concept {

  public static final Set emptySet = new HashedSet();

  /** Return whether this Concept is extensional. */
  public boolean isExtensional() {
    return false;
  }

  /** Return whether this Concept restricts which features are allowed.
   *  A non-restricted Concept is appropriate for every feature.
   */
  public boolean isRestricted() {
      return false; // does not allow any features, so it is restricted!
  }

  /** Look for the most general common extensional supertype of this and "that". */
  public ConceptEnumeration extensionalSuperconcepts() {
     return null; // not extensional, should never be called!
  }

  /** Return whether the feature <feature> is appropriate in this Concept. 
   */
  public boolean isApprop(Name feature) {
      return false;  // has no Features
  }

  /** Return the required ConceptSet for the Value under the given feature. */
  public ConceptSet appropConcepts(Name featureName) throws NoSuchElementException {
    throw new NoSuchElementException();  // has no Features
  }   

  /** Return the required Type of the Value found under the given feature. */
  public Type appropType(Name featureName) throws NoSuchElementException {
    throw new NoSuchElementException();  // has no Features
  }


  /** Return an Enumeration of all appropriate features. */
  public Enumeration appropFeatureNames() {
    return emptySet.elements();  // has no Features
  }

  public ConceptSet unify(Concept that) throws UnificationFailure {
      if (isNotA(that))
	  throw new UnificationFailure();
      ConceptSet result=new ConceptSet();
      if (isa(that)) {
	  result.addConcept(this);
      } else {
	  result.addConcept(that);
	  if (!that.isa(this))
	      result.addConcept(this);
      }
      return result;
  }

  public String toString() {
      return getName();
  }

}
