package de.uni_hamburg.fs;

import collections.*;
import java.util.*;

public interface Type  {

  /** The most general Type. */
  public static final Type TOP = new ConjunctiveType();

  /** Return the name of this Type. */
  public String getName();

  /** Return whether this Type is extensional. */
  public boolean isExtensional();

  /** Return whether the feature <feature> is appropriate in this Type. */
  public boolean isApprop(Name featureName);

  /** Return the required Type of the Value found under the given feature.
    * The feature has to be appropriate for this Type.
    */
  public Type appropType(Name featureName) throws NoSuchElementException;

  /** Return an Enumeration of all appropriate features. */
  public Enumeration appropFeatureNames();

  /** Return whether this Type subsumes <that> Type.
    * In other words, return wheter this Type is more general than <that> Type.
    */
  public boolean subsumes(Type that);

  /** Return whether this Type is subsumed by <that> Type.
    * In other words, return wheter this Type is more special than <that>
    * Type.
    */
  public boolean subsumedBy(Type that);

  /** Return the unification of this Type and <that> Type.
    * this Type is not modified!
    */
  public Type unify(Type that) throws UnificationFailure;

  /** Return whether this Type and <that> Type are compatible.
    */
  public boolean canUnify(Type that);

  /** Look for the most general common extensional supertype of this and <that>. */
  public Type mostGeneralExtensionalSupertype(Type that);

  /** Return an Enumeration of all Concepts this Type is 
   *   composed of.
   */
  public CollectionEnumeration concepts();

  // should be unnecessary, Renew bug has to be fixed:
  public boolean equals(Object that);

}
